@@ -10,13 +10,12 @@ from sql.operators import Concat
 
 from trytond.model import Workflow, ModelView, ModelSQL, fields
 from trytond.modules.company import CompanyReport
-from trytond.wizard import Wizard, StateTransition, StateView, StateAction, \
-    Button
+from trytond.wizard import Wizard, StateTransition, StateView, Button
 from trytond import backend
 from trytond.pyson import Eval, Not, Equal, If, Or, And, Bool, In, Get, Id
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
-from trytond.tools import reduce_ids
+from trytond.tools import reduce_ids, grouped_slice
 
 __all__ = ['ShipmentIn', 'ShipmentInReturn',
     'ShipmentOut', 'ShipmentOutReturn',
@@ -25,7 +24,6 @@ __all__ = ['ShipmentIn', 'ShipmentInReturn',
     'Address',
     'AssignShipmentInternalAssignFailed', 'AssignShipmentInternal',
     'AssignShipmentInReturnAssignFailed', 'AssignShipmentInReturn',
-    'CreateShipmentOutReturn',
     'DeliveryNote', 'PickingList',
     'SupplierRestockingList', 'CustomerReturnRestockingList',
     'InteralShipmentReport']
@@ -235,8 +233,7 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
             for company_id, values in itertools.groupby(cursor.fetchall(),
                     operator.itemgetter(1)):
                 shipment_ids = [x[0] for x in values]
-                for i in range(0, len(shipment_ids), cursor.IN_MAX):
-                    sub_ids = shipment_ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(shipment_ids):
                     red_sql = reduce_ids(sql_table.id, sub_ids)
                     cursor.execute(*sql_table.update(
                             columns=[sql_table.company],
@@ -359,14 +356,9 @@ class ShipmentIn(Workflow, ModelSQL, ModelView):
                     'planned_date': inventory_date,
                     })
 
-    @classmethod
-    def get_origins(cls, shipments, name):
-        origins = {}
-        with Transaction().set_user(0, set_context=True):
-            for shipment in cls.browse(shipments):
-                origins[shipment.id] = ', '.join(set(itertools.ifilter(None,
-                            (m.origin_name for m in shipment.moves))))
-        return origins
+    def get_origins(self, name):
+        return ', '.join(set(itertools.ifilter(None,
+                    (m.origin_name for m in self.moves))))
 
     @classmethod
     def create(cls, vlist):
@@ -618,8 +610,7 @@ class ShipmentInReturn(Workflow, ModelSQL, ModelView):
             for company_id, values in itertools.groupby(cursor.fetchall(),
                     operator.itemgetter(1)):
                 shipment_ids = [x[0] for x in values]
-                for i in range(0, len(shipment_ids), cursor.IN_MAX):
-                    sub_ids = shipment_ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(shipment_ids):
                     red_sql = reduce_ids(sql_table.id, sub_ids)
                     cursor.execute(*sql_table.update(
                             columns=[sql_table.company],
@@ -659,14 +650,9 @@ class ShipmentInReturn(Workflow, ModelSQL, ModelView):
                     'planned_date': shipment._move_planned_date,
                     })
 
-    @classmethod
-    def get_origins(cls, shipments, name):
-        origins = {}
-        with Transaction().set_user(0, set_context=True):
-            for shipment in cls.browse(shipments):
-                origins[shipment.id] = ', '.join(set(itertools.ifilter(None,
-                            (m.origin_name for m in shipment.moves))))
-        return origins
+    def get_origins(self, name):
+        return ', '.join(set(itertools.ifilter(None,
+                    (m.origin_name for m in self.moves))))
 
     @classmethod
     def create(cls, vlist):
@@ -948,8 +934,7 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
             for company_id, values in itertools.groupby(cursor.fetchall(),
                     operator.itemgetter(1)):
                 shipment_ids = [x[0] for x in values]
-                for i in range(0, len(shipment_ids), cursor.IN_MAX):
-                    sub_ids = shipment_ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(shipment_ids):
                     red_sql = reduce_ids(sql_table.id, sub_ids)
                     cursor.execute(*sql_table.update(
                             columns=[sql_table.company],
@@ -1043,15 +1028,9 @@ class ShipmentOut(Workflow, ModelSQL, ModelView):
                 'moves': value,
                 })
 
-    @classmethod
-    def get_origins(cls, shipments, name):
-        origins = {}
-        with Transaction().set_user(0, set_context=True):
-            for shipment in cls.browse(shipments):
-                origins[shipment.id] = ', '.join(set(itertools.ifilter(None,
-                            (m.origin_name for m in shipment.moves))))
-        return origins
-
+    def get_origins(self, name):
+        return ', '.join(set(itertools.ifilter(None,
+                    (m.origin_name for m in self.moves))))
     @classmethod
     @ModelView.button
     @Workflow.transition('draft')
@@ -1418,7 +1397,7 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
         cls._transitions |= set((
                 ('draft', 'received'),
                 ('received', 'done'),
-                ('received', 'draf'),
+                ('received', 'draft'),
                 ('draft', 'cancel'),
                 ('received', 'cancel'),
                 ('cancel', 'draft'),
@@ -1474,8 +1453,7 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
             for company_id, values in itertools.groupby(cursor.fetchall(),
                     operator.itemgetter(1)):
                 shipment_ids = [x[0] for x in values]
-                for i in range(0, len(shipment_ids), cursor.IN_MAX):
-                    sub_ids = shipment_ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(shipment_ids):
                     red_sql = reduce_ids(sql_table.id, sub_ids)
                     cursor.execute(*sql_table.update(
                             columns=[sql_table.company],
@@ -1594,14 +1572,9 @@ class ShipmentOutReturn(Workflow, ModelSQL, ModelView):
                     'planned_date': inventory_date,
                     })
 
-    @classmethod
-    def get_origins(cls, shipments, name):
-        origins = {}
-        with Transaction().set_user(0, set_context=True):
-            for shipment in cls.browse(shipments):
-                origins[shipment.id] = ', '.join(set(itertools.ifilter(None,
-                            (m.origin_name for m in shipment.moves))))
-        return origins
+    def get_origins(self, name):
+        return ', '.join(set(itertools.ifilter(None,
+                    (m.origin_name for m in self.moves))))
 
     @classmethod
     def create(cls, vlist):
@@ -1810,7 +1783,8 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
                 | ~Eval('from_location') | ~Eval('to_location')),
             },
         domain=[
-            ('from_location', '=', Eval('from_location')),
+            ('from_location', 'child_of', [Eval('from_location', -1)],
+                'parent'),
             ('to_location', '=', Eval('to_location')),
             ('company', '=', Eval('company')),
             ],
@@ -1910,8 +1884,7 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
             for company_id, values in itertools.groupby(cursor.fetchall(),
                     operator.itemgetter(1)):
                 shipment_ids = [x[0] for x in values]
-                for i in range(0, len(shipment_ids), cursor.IN_MAX):
-                    sub_ids = shipment_ids[i:i + cursor.IN_MAX]
+                for sub_ids in grouped_slice(shipment_ids):
                     red_sql = reduce_ids(sql_table.id, sub_ids)
                     cursor.execute(*sql_table.update(
                             columns=[sql_table.company],
@@ -2010,8 +1983,9 @@ class ShipmentInternal(Workflow, ModelSQL, ModelView):
     @ModelView.button
     def assign_try(cls, shipments):
         Move = Pool().get('stock.move')
-        if Move.assign_try([m for s in shipments
-                    for m in s.moves]):
+        to_assign = [m for s in shipments for m in s.moves
+            if m.from_location.type != 'lost_found']
+        if not to_assign or Move.assign_try(to_assign):
             cls.assign(shipments)
             return True
         else:
@@ -2126,66 +2100,6 @@ class AssignShipmentInReturn(Wizard):
         return 'end'
 
 
-class CreateShipmentOutReturn(Wizard):
-    'Create Customer Return Shipment'
-    __name__ = 'stock.shipment.out.return.create'
-    start = StateAction('stock.act_shipment_out_return_form')
-
-    @classmethod
-    def __setup__(cls):
-        super(CreateShipmentOutReturn, cls).__setup__()
-        cls._error_messages.update({
-                'shipment_done_title': 'You can not create return shipment',
-                'shipment_done_msg': ('The shipment with code "%s" is not yet '
-                    'sent.'),
-                })
-
-    def do_start(self, action):
-        pool = Pool()
-        ShipmentOut = pool.get('stock.shipment.out')
-        ShipmentOutReturn = pool.get('stock.shipment.out.return')
-
-        shipment_ids = Transaction().context['active_ids']
-        shipment_outs = ShipmentOut.browse(shipment_ids)
-
-        to_create = []
-        for shipment_out in shipment_outs:
-            if shipment_out.state != 'done':
-                self.raise_user_error('shipment_done_title',
-                        error_description='shipment_done_msg',
-                        error_description_args=shipment_out.code)
-
-            incoming_moves = []
-            moves_to_create = []
-            for move in shipment_out.outgoing_moves:
-                moves_to_create.append({
-                        'product': move.product.id,
-                        'quantity': move.quantity,
-                        'uom': move.uom.id,
-                        'from_location': move.to_location.id,
-                        'to_location': (
-                            shipment_out.warehouse.input_location.id),
-                        'company': move.company.id,
-                        })
-            if moves_to_create:
-                incoming_moves.append(('create', moves_to_create))
-            to_create.append({
-                    'customer': shipment_out.customer.id,
-                    'delivery_address': shipment_out.delivery_address.id,
-                    'warehouse': shipment_out.warehouse.id,
-                    'incoming_moves': incoming_moves,
-                    })
-        shipment_out_returns = ShipmentOutReturn.create(to_create)
-
-        data = {'res_id': [x.id for x in shipment_out_returns]}
-        if len(shipment_out_returns) == 1:
-            action['views'].reverse()
-        return action, data
-
-    def transition_start(self):
-        return 'end'
-
-
 class DeliveryNote(CompanyReport):
     'Delivery Note'
     __name__ = 'stock.shipment.out.delivery_note'
